#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Description: Test for nss-pam-ldapd with nss db
#   Author: David Spurek <dspurek@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/bin/rhts-environment.sh
. /usr/lib/beakerlib/beakerlib.sh

PACKAGES=( "pam_ldap"         \
           "nss-pam-ldapd"    \
           "nss-tools"        \
           "authconfig"       \
           "openldap"         \
           "openldap-clients" \
           "openldap-servers" \
           "rsyslog"          )

CONFIG_FILE="pam_ldap"
LDAP_SERVICE="slapd"
LOG_SERVICE="rsyslog"                    

BDN='cn=Manager,dc=my-domain,dc=com'
PWD='x'

rlJournalStart

  rlPhaseStartSetup

      yum clean all
      for P in ${PACKAGES[@]}; do
        rlCheckRpm $P || rlRun "yum -y install $P" 0 "Installing $P"
      done


	      rlServiceStop "nscd"
	      rlServiceStop "nslcd"


	rlServiceStop $LOG_SERVICE
	rlServiceStop $LDAP_SERVICE

	rlFileBackup "/etc/$LOG_SERVICE.conf"    
	rlFileBackup "/etc/sysconfig/iptables"
	rlFileBackup "/etc/$CONFIG_FILE.secret"
	rlFileBackup "/etc/$CONFIG_FILE.conf"     
	rlFileBackup "/etc/sysconfig/ldap"      
	rlFileBackup "/etc/openldap/slapd.conf"
	rlFileBackup "/etc/openldap/ldap.conf"      
	rlFileBackup "/etc/openldap/slapd.d" && rm -rf /etc/openldap/slapd.d/*
	rlFileBackup "/var/lib/ldap/" && rm -rf /var/lib/ldap/*
	rlFileBackup "/etc/openldap/cacerts/" && rlRun "rm -rf /etc/openldap/cacerts/*" 0
	rlFileBackup "/etc/openldap/slapd.d/" && rlRun "rm -rf /etc/openldap/slapd.d/*" 0
	rlFileBackup "/var/run/openldap/" && rlRun "rm -rf /var/run/openldap/*" 0
	rlFileBackup "/etc/hosts" 
	rlFileBackup "/etc/nsswitch.conf"             

	rlRun "cp -f nsswitch.conf /etc/nsswitch.conf" 0
	rlRun "cp -f ldap /etc/sysconfig/ldap" 0
	rlRun "echo '127.0.0.1 my-domain.com' >> /etc/hosts" 0
	rlRun "cp -f *.pem /etc/openldap/cacerts/" 0
	rlRun "chown ldap:ldap /etc/openldap/cacerts/*" 0
	rlRun "chmod 444 /etc/openldap/cacerts/*" 0 
 
	rlRun "cacertdir_rehash /etc/openldap/cacerts/" 0    
	rlRun "mkdir /tmp/pam_ldap-nssdb && certutil -d /tmp/pam_ldap-nssdb -A -n 'CA cert' -t CT,, -a -i cacert.pem" 0

	cat /etc/openldap/ldap.conf | grep -v "TLS_CACERTDIR"  > /tmp/ldap.conf
	cp /tmp/ldap.conf /etc/openldap/ldap.conf 
	rlRun "echo 'TLS_CACERTDIR /tmp/pam_ldap-nssdb' >> /etc/openldap/ldap.conf" 0
	rm -f /tmp/ldap.conf
   
                      
      rlRun "cp -f slapd.conf /etc/openldap/slapd.conf" 0
	      slaptest -f /etc/openldap/slapd.conf -F /etc/openldap/slapd.d/
	      rlRun "chown -R ldap:ldap /etc/openldap/slapd.d/" 0

      # Loading testing data into OpenLDAP servers.
      rlRun "slapadd -l data.ldif" 0

      rlRun "chown -R ldap:ldap /var/lib/ldap" 0
      
          # Enable logging.

      rlRun "echo 'local4.* /var/log/slapd.log' >> /etc/rsyslog.conf" 0
      
      rlRun "restorecon -Rv /etc/"   

	rlRun "service $LOG_SERVICE start && sleep 5" 0
	rlRun "service $LDAP_SERVICE start && sleep 10" 0

        
    rlPhaseEnd

    rlPhaseStartTest "Test pam_ldap with TLS" 

	rlRun "authconfig --enableldap --enableldaptls --enableldapauth --updateall \
	--ldapbasedn dc=my-domain,dc=com --ldapserver ldap://my-domain.com" 0 \
	"Configuring authentication mechanisms"   


	cat /etc/nslcd.conf | grep -v "ssl" | grep -v "tls_cacertdir" | grep -v "tls_cacertfile" > /tmp/nslcd_nossl.conf
	cp /tmp/nslcd_nossl.conf /etc/nslcd.conf
	rlRun "echo 'ssl start_tls' >> /etc/nslcd.conf" 0
	rlRun "echo 'tls_reqcert demand' >> /etc/nslcd.conf" 0 
      rlRun "echo 'tls_cacertdir /tmp/pam_ldap-nssdb' >> /etc/nslcd.conf" 0
	rlRun "echo 'binddn cn=Manager,dc=my-domain,dc=com' >> /etc/nslcd.conf" 0
	rlRun "echo 'bindpw x' >> /etc/nslcd.conf" 0
	rlRun "service nslcd restart & sleep 5" 0


	cp  /etc/$CONFIG_FILE.conf /tmp/${CONFIG_FILE}_tls.conf
	cp  /etc/$CONFIG_FILE.secret /tmp/${CONFIG_FILE}_tls.secret     
	cat /etc/$CONFIG_FILE.conf | grep -v "ssl" > /tmp/${CONFIG_FILE}_nossl.conf
	cp /tmp/${CONFIG_FILE}_nossl.conf /etc/$CONFIG_FILE.conf
	rlRun "echo 'ssl start_tls' >> /etc/$CONFIG_FILE.conf" 0                
	rlRun "echo 'binddn uid=binddn,ou=people,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'bindpw x' >> /etc/$CONFIG_FILE.conf" 0 
	rlRun "echo 'rootbinddn cn=Manager,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'x' > /etc/$CONFIG_FILE.secret" 0
	rlRun "chmod 0600 /etc/$CONFIG_FILE.secret" 0    

	cat /etc/$CONFIG_FILE.conf | grep -v "tls_cacertdir" | grep -v "tls_cacertfile"  > /tmp/${CONFIG_FILE}_notls_cacertdir.conf
	cp /tmp/${CONFIG_FILE}_notls_cacertdir.conf /etc/$CONFIG_FILE.conf
	rm -f /tmp/${CONFIG_FILE}_notls_cacertdir.conf


	rlRun "echo 'sslpath /tmp/pam_ldap-nssdb' >> /etc/$CONFIG_FILE.conf" 0 


	cat /etc/openldap/ldap.conf | grep -v "TLS_CACERTDIR"  > /tmp/ldap.conf
	cp /tmp/ldap.conf /etc/openldap/ldap.conf 
	rlRun "echo 'TLS_CACERTDIR /tmp/pam_ldap-nssdb' >> /etc/openldap/ldap.conf" 0
	rm -f /tmp/ldap.conf

	rlRun "ldapsearch -H ldap://my-domain.com -x -ZZ '*'" 0
	rlRun "getent passwd ldapuser" 0

    rlPhaseEnd
    
  rlPhaseStartCleanup


#		rlRun "authconfig --restorebackup backup"
#	rlRun "service $LDAP_SERVICE stop && sleep 10" 0

#	rlRun "rm -rf /etc/openldap/slapd*" 0 

#	# Delete test certificates
#	rlRun "rm -f /etc/openldap/cacerts/*.pem" 0

#	# Delete test files
#	rlRun "rm -rf /var/lib/ldap/*" 0
#	rlRun "rm -f /etc/openldap/slapd.conf" 0
#	rlRun "rm -rf /var/run/openldap/*" 0   
#	rlRun "rm -f /var/log/slapd.log" 0

#	rm -r /home/ldapuser
#	rm -f /tmp/$CONFIG_FILE.conf /tmp/$CONFIG_FILE.secret /tmp/${CONFIG_FILE}_passwd.conf /tmp/${CONFIG_FILE}_tls.conf /tmp/${CONFIG_FILE}_tls.secret
#	rm -r /tmp/${CONFIG_FILE}_ssl.conf /tmp/${CONFIG_FILE}_ssl.secret
#	rm -rf /tmp/$CONFIG_FILE-nssdb

#	rlFileRestore



#		rlServiceRestore "nscd"
#		rlServiceRestore "nslcd"
#	rlServiceRestore $LOG_SERVICE
#	rlServiceRestore $LDAP_SERVICE

  rlPhaseEnd

rlJournalPrintText

rlJournalEnd






